/* Somaplayer - Copyright (C) 2003-5 bakunin - Andrea Marchesini 
 *                                     <bakunin@autistici.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * This program is released under the GPL with the additional exemption that
 * compiling, linking, and/or using OpenSSL is allowed.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#else
# error Use configure; make; make install
#endif

#include "renderix.h"

static const char *credits[] = {
  "Andrea Marchesini aka bakunin\n<bakunin@autistici.org>\nCoder and Mantainer",
  "kino\n<kino@kino3d.com>\ngraphics",
  NULL
};

static int about_t = 0;
static int about_timer = 0;
static GtkWidget *about_text = NULL;
static GtkWidget *about_window = NULL;

static gint about_timeout (void);
static void about_close (GtkWidget *);
static void about_button_press_event (GtkWidget *, GdkEventButton *);

void
about_show (void)
{
  static gboolean showed = FALSE;

  if (showed == FALSE)
    {
      DEBUG ("about showed");

      gtk_widget_show_all (about_window);
      showed = TRUE;
    }
  else
    {
      DEBUG ("about hided");

      gtk_widget_hide (about_window);
      showed = FALSE;
    }
}

void
about_init (void)
{
  GtkWidget *box;
  GdkPixbuf *pixbuf;
  GtkWidget *image;
  GtkWidget *scrolledwindow;
  GtkWidget *textarea;
  GtkWidget *button;

  DEBUG ("about init");

  about_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (about_window), "About...");
  gtk_widget_realize (about_window);

  box = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (box);
  gtk_container_add (GTK_CONTAINER (about_window), box);

  pixbuf = gdk_pixbuf_new_from_file (PATH_IMAGE, NULL);
  image = gtk_image_new_from_pixbuf (pixbuf);
  gtk_widget_show (image);

  gtk_box_pack_start (GTK_BOX (box), image, TRUE, TRUE, 0);

  scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow);
  gtk_box_pack_start (GTK_BOX (box), scrolledwindow, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow),
				  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_widget_set_size_request (scrolledwindow, -1, 100);

  textarea = gtk_text_view_new ();
  gtk_widget_show (textarea);
  gtk_container_add (GTK_CONTAINER (scrolledwindow), textarea);
  gtk_text_view_set_editable (GTK_TEXT_VIEW (textarea), FALSE);
  gtk_text_view_set_justification (GTK_TEXT_VIEW (textarea),
				   GTK_JUSTIFY_CENTER);
  gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (textarea), FALSE);
  gtk_text_view_set_pixels_above_lines (GTK_TEXT_VIEW (textarea), 5);
  gtk_text_view_set_pixels_below_lines (GTK_TEXT_VIEW (textarea), 5);
  gtk_text_view_set_left_margin (GTK_TEXT_VIEW (textarea), 5);
  gtk_text_view_set_right_margin (GTK_TEXT_VIEW (textarea), 5);

  about_text = (GtkWidget *) gtk_text_view_get_buffer (GTK_TEXT_VIEW (textarea));
  gtk_text_buffer_create_tag (GTK_TEXT_BUFFER (about_text), "text", "weight",
			      PANGO_WEIGHT_BOLD, "scale",
			      PANGO_SCALE_MEDIUM, NULL);

  g_signal_connect ((gpointer) about_window, "delete_event",
		    G_CALLBACK (about_show), NULL);

  gtk_signal_connect_object (GTK_OBJECT (textarea), "button_press_event",
			     GTK_SIGNAL_FUNC
			     (about_button_press_event), textarea);

  button = gtk_button_new_with_mnemonic ("Close");
  gtk_widget_show (button);
  gtk_box_pack_start (GTK_BOX (box), button, FALSE, TRUE, 0);

  g_signal_connect ((gpointer) button, "clicked",
		    G_CALLBACK (about_close), NULL);

  about_timer = 0;
  about_timeout ();
  about_t = g_timeout_add (2000, (GSourceFunc) about_timeout, NULL);
}

static void
about_button_press_event (GtkWidget * w, GdkEventButton * event)
{

  DEBUG ("about press event");

  if (event->button == 3 && about_timer > 1)
    about_timer -= 2;

  g_source_remove (about_t);
  about_timeout ();
  about_t = g_timeout_add (2000, (GSourceFunc) about_timeout, NULL);
}

/* Timeout for scrolling */
static gint
about_timeout (void)
{
  GtkTextIter start;
  GtkTextIter end;

  DEBUG ("about timeout");

  gtk_text_buffer_get_start_iter (GTK_TEXT_BUFFER (about_text), &start);
  gtk_text_buffer_get_end_iter (GTK_TEXT_BUFFER (about_text), &end);

  gtk_text_buffer_delete (GTK_TEXT_BUFFER (about_text), &start, &end);

  gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (about_text), &start, 0);

  if (!credits[about_timer])
    about_timer = 0;

  gtk_text_buffer_insert_with_tags_by_name (GTK_TEXT_BUFFER (about_text),
					    &start, credits[about_timer],
					    -1, "text", NULL);
  about_timer++;

  return TRUE;
}

static void
about_close (GtkWidget * w)
{
  DEBUG ("about close");

  about_show ();
}

/* EOF */
