/* SomaX - Copyright (C) 2005 bakunin - Andrea Marchesini 
 *                                <bakunin@autistici.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#else
# error Use configure; make; make install
#endif

#include "renderix.h"

#define GRAPHIC_MAX 20

static gboolean graphic_expose_event (GtkWidget *, GdkEventExpose *,
				      gpointer);
static void graphic_refresh_true (GtkWidget *, gboolean);

GtkWidget *
graphic_new (void)
{
  GtkWidget *graphic;

  DEBUG ("graphic new");

  graphic = gtk_drawing_area_new ();

  g_signal_connect ((gpointer) graphic, "expose_event",
		    G_CALLBACK (graphic_expose_event), NULL);

  gtk_widget_set_events (graphic, gtk_widget_get_events (graphic) |
			 GDK_BUTTON_PRESS_MASK);

  return graphic;
}

void
graphic_set_range (GtkWidget * graphic, gint range)
{
  DEBUG ("graphic set range");

  gtk_object_set_data (GTK_OBJECT (graphic), "range", (gpointer) range);

  if (GTK_WIDGET_VISIBLE (graphic))
    graphic_refresh_true (graphic, TRUE);
}

void
graphic_set_color (GtkWidget * graphic, gchar * color)
{
  GdkColor *data;

  DEBUG ("graphic set color");

  data = gtk_object_get_data (GTK_OBJECT (graphic), "color");

  if (!data)
    data = g_malloc (sizeof (GdkColor));

  gdk_color_parse (color, data);
  gtk_object_set_data (GTK_OBJECT (graphic), "color", data);

  if (GTK_WIDGET_VISIBLE (graphic))
    graphic_refresh_true (graphic, TRUE);
}

void
graphic_set_data (GtkWidget * graphic, gint data)
{
  GList *list;
  gint range;
  gint i;

  DEBUG ("graphic set data");

  list = (GList *) gtk_object_get_data (GTK_OBJECT (graphic), "data");
  range = (gint) gtk_object_get_data (GTK_OBJECT (graphic), "range");

  list = g_list_append (list, (gpointer) data);

  if (range < g_list_length (list))
    {
      i = g_list_length (list) - range;

      for (; i; i--)
	list = g_list_remove (list, list->data);
    }

  gtk_object_set_data (GTK_OBJECT (graphic), "data", list);

  if (GTK_WIDGET_VISIBLE (graphic))
    graphic_refresh_true (graphic, TRUE);
}

static gboolean
graphic_expose_event (GtkWidget * graphic, GdkEventExpose * event,
		      gpointer dummy)
{
  DEBUG ("graphic expose");

  graphic_refresh_true (graphic, FALSE);

  return TRUE;
}

static void
graphic_refresh_true (GtkWidget * graphic, gboolean done)
{
  PangoLayout *layout;
  PangoAttribute *attr;
  PangoAttrList *list_attr;
  int pango_height, pango_width;
  int width, height;
  int x, y;
  int k, i;
  char s[1024];
  gint range;
  gint size;
  GList *list;
  GdkGC *gc;
  GdkColor *color;
  gboolean text;

  DEBUG ("graphic refresh");

  width = graphic->allocation.width - (GRAPHIC_MAX * 2);
  height = graphic->allocation.height - (GRAPHIC_MAX * 2);
  x = GRAPHIC_MAX * 2;
  y = GRAPHIC_MAX * 2;

  gdk_window_clear (graphic->window);

  gdk_draw_rectangle (graphic->window,
		      graphic->style->light_gc[graphic->state], TRUE, 0, 0,
		      graphic->allocation.width, graphic->allocation.height);

  /* line x */
  gdk_draw_line (graphic->window, graphic->style->black_gc, x, height, width,
		 height);

  /* line y */
  gdk_draw_line (graphic->window, graphic->style->black_gc, x, y, x, height);

  /* Cpu flags  */
  list_attr = pango_attr_list_new ();
  attr = pango_attr_scale_new (PANGO_SCALE_X_SMALL);
  pango_attr_list_insert (list_attr, attr);
  attr->start_index = 0;
  attr->end_index = -1;

  k = (height - y) / 10;
  size = k;

  for (i = 0; i < 11; i++)
    {
      gdk_draw_line (graphic->window, graphic->style->black_gc, x - 5,
		     height - (k * i), x, height - (k * i));

      snprintf (s, sizeof (s), "%d%%", i * 10);
      layout = gtk_widget_create_pango_layout (graphic, s);
      pango_layout_set_attributes (layout, list_attr);
      pango_layout_get_pixel_size (layout, &pango_width, &pango_height);

      gtk_paint_layout (graphic->style, graphic->window, GTK_STATE_NORMAL,
			TRUE, NULL, graphic, NULL, x - pango_width - 8,
			height - (k * i) - (pango_height / 2), layout);

      g_object_unref (layout);

    }

  pango_attr_list_unref (list_attr);

  /* range && data */
  range = (gint) gtk_object_get_data (GTK_OBJECT (graphic), "range");
  list = (GList *) gtk_object_get_data (GTK_OBJECT (graphic), "data");
  color = (GdkColor *) gtk_object_get_data (GTK_OBJECT (graphic), "color");

  list_attr = pango_attr_list_new ();
  attr = pango_attr_scale_new (PANGO_SCALE_X_SMALL);
  pango_attr_list_insert (list_attr, attr);
  attr->start_index = 0;
  attr->end_index = -1;
  k = (width - x) / range;

  snprintf (s, sizeof (s), "%d", i + 1);
  layout = gtk_widget_create_pango_layout (graphic, s);
  pango_layout_set_attributes (layout, list_attr);
  pango_layout_get_pixel_size (layout, &pango_width, &pango_height);
  g_object_unref (layout);

  if (pango_width < k)
    text = TRUE;
  else
    text = FALSE;

  for (i = 0; i < range + 1; i++)
    {
      gdk_draw_line (graphic->window, graphic->style->black_gc, x + (k * i),
		     height, x + (k * i), height + 5);

      if (text == TRUE && i != range)
	{
	  snprintf (s, sizeof (s), "%d", i + 1);
	  layout = gtk_widget_create_pango_layout (graphic, s);
	  pango_layout_set_attributes (layout, list_attr);
	  pango_layout_get_pixel_size (layout, &pango_width, &pango_height);

	  gtk_paint_layout (graphic->style, graphic->window, GTK_STATE_NORMAL,
			    TRUE, NULL, graphic, NULL,
			    x + (k * i) + (k / 2) - (pango_width / 2),
			    height + pango_height, layout);

	  g_object_unref (layout);
	}

      gc = gdk_gc_new (graphic->window);
      if (color)
	gdk_gc_set_rgb_fg_color (gc, color);
      else
	{
	  GdkColor col_tmp;
	  gdk_color_parse ("#614CAB", &col_tmp);
	  gdk_gc_set_rgb_fg_color (gc, &col_tmp);
	}

      if (list)
	{
	  gdk_draw_rectangle (graphic->window, gc, TRUE, x + (k * i) + 1,
			      height - (size / 10) * (gint) list->data, k - 1,
			      (size / 10) * (gint) list->data);

	  gdk_draw_line (graphic->window, graphic->style->black_gc,
			 x + (k * (i + 1)),
			 height - (size / 10) * (gint) list->data,
			 x + (k * (i + 1)), height);
	  gdk_draw_line (graphic->window, graphic->style->black_gc,
			 x + (k * i) + 1,
			 height - (size / 10) * (gint) list->data,
			 x + (k * (i + 1)),
			 height - (size / 10) * (gint) list->data);

	  list = list->next;
	}

      gdk_gc_unref (gc);

    }

  if (done == TRUE)
    gtk_widget_queue_draw (graphic);

  DEBUG ("graphic refresh - close");
}

/* EOF */
