/* SomaX - Copyright (C) 2005 bakunin - Andrea Marchesini 
 *                                <bakunin@autistici.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#else
# error Use configure; make; make install
#endif

#include "renderix.h"

#define SIZE_100        1
#define SIZE_75         2
#define SIZE_50         3
#define SIZE_25         4
#define SIZE_125        5
#define SIZE_150        6
#define SIZE_200        7
#define SIZE_400        8
#define SIZE_AUTO       9
#define SIZE_WIDTH      10
#define SIZE_HEIGHT     11

static void image_insert (char *, char *, struct renderix_data *);
static void image_zoom (char *);
static GtkWidget *image_zoom_popup (struct renderix_dialog_data *);
static gboolean image_zoom_popup_show (GtkWidget *, GdkEventButton *,
				       gpointer);
static GtkWidget *image_create_image (GtkWidget *, char *, gint);
static void image_zoom_set (GtkWidget *, int);

void
image_new (char *buf, char *time, struct renderix_data *data)
{
  struct stat st;
  char s[1024];
  int i = 0;

  DEBUG ("image new");

  while (*buf == ' ' || *buf == '\t')
    buf++;

  while (*buf && i < sizeof (s) - 1)
    {
      if (*buf == ' ' || *buf == '\t')
	break;
      s[i++] = *buf;
      buf++;
    }
  s[i] = 0;

  if (*s && !lstat (s, &st))
    image_insert (s, time, data);
}

static void
image_insert (char *buf, char *time, struct renderix_data *data)
{
  GtkListStore *store;
  GtkTreeIter iter;
  int i;

  DEBUG ("image insert");

  for (i = strlen (buf) - 1; i; i--)
    {
      if (buf[i] == '/')
	{
	  i++;
	  break;
	}
    }

  store =
    (GtkListStore *) gtk_tree_view_get_model (GTK_TREE_VIEW (data->images));

  gtk_list_store_append (store, &iter);
  gtk_list_store_set (store, &iter, 0, buf, 1, buf + i, 2, time, -1);
}

gboolean
image_select (GtkWidget * w, GdkEventButton * event, gpointer dummy)
{
  GtkTreeSelection *selection;
  GtkTreeModel *model;
  GtkTreeIter iter;
  char *file;

  DEBUG ("image select");

  if (event->type != GDK_2BUTTON_PRESS)
    return FALSE;

  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (w));
  if (gtk_tree_selection_get_selected (selection, &model, &iter) == FALSE)
    return FALSE;

  gtk_tree_model_get (model, &iter, 0, &file, -1);

  if (!file)
    return FALSE;

  image_zoom (file);
  return TRUE;
}

static void
image_zoom (char *file)
{
  GtkWidget *dialog;
  GtkWidget *sw;
  GtkWidget *vp;
  GtkWidget *vbox;
  GtkWidget *button;
  GtkWidget *image;
  GtkWidget *label;
  GtkWidget *popup;

  struct renderix_dialog_data d_data;
  int ok;

  DEBUG ("image zoom");

  dialog = gtk_dialog_new ();
  gtk_widget_set_size_request (dialog, 600, 400);
  gtk_window_set_title (GTK_WINDOW (dialog), file);
  gtk_window_set_type_hint (GTK_WINDOW (dialog), GDK_WINDOW_TYPE_HINT_DIALOG);
  gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);

  d_data.dialog = dialog;
  d_data.size = SIZE_AUTO;

  popup = image_zoom_popup (&d_data);

  vbox = gtk_vbox_new (FALSE, 8);
  gtk_widget_show (vbox);
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), vbox, TRUE, TRUE,
		      0);

  label = gtk_label_new (file);
  gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);

  sw = gtk_scrolled_window_new (NULL, NULL);
  gtk_box_pack_start (GTK_BOX (vbox), sw, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
				  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  vp = gtk_viewport_new (NULL, NULL);
  gtk_container_add (GTK_CONTAINER (sw), vp);
  gtk_viewport_set_shadow_type (GTK_VIEWPORT (vp), GTK_SHADOW_NONE);

  gtk_signal_connect_object (GTK_OBJECT (vp), "button_press_event",
			     GTK_SIGNAL_FUNC (image_zoom_popup_show), popup);

  image = image_create_image (vp, file, SIZE_100);
  gtk_container_add (GTK_CONTAINER (vp), image);

  button = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (button);
  gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button, GTK_RESPONSE_OK);

  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);

  gtk_widget_show_all (dialog);
  while (1)
    {
      ok = 0;

      switch (gtk_dialog_run (GTK_DIALOG (dialog)))
	{
	case GTK_RESPONSE_CANCEL:	/* Cancel is resize image */
	  gtk_container_remove (GTK_CONTAINER (vp), image);

	  image = image_create_image (vp, file, d_data.size);
	  gtk_container_add (GTK_CONTAINER (vp), image);
	  gtk_widget_show (image);

	  break;
	default:
	  ok = 1;
	  break;
	}

      if (ok)
	break;
    }

  gtk_widget_destroy (dialog);
}

static GtkWidget *
image_zoom_popup (struct renderix_dialog_data *data)
{
  GtkWidget *menu;
  GtkWidget *size;

  DEBUG ("image zoom popup");

  menu = gtk_menu_new ();

  size = gtk_menu_item_new_with_mnemonic ("25%");
  gtk_object_set_data (GTK_OBJECT (size), "data", data);
  gtk_widget_show (size);
  gtk_container_add (GTK_CONTAINER (menu), size);
  g_signal_connect ((gpointer) size, "activate",
		    G_CALLBACK (image_zoom_set), (void *) SIZE_25);

  size = gtk_menu_item_new_with_mnemonic ("50%");
  gtk_object_set_data (GTK_OBJECT (size), "data", data);
  gtk_widget_show (size);
  gtk_container_add (GTK_CONTAINER (menu), size);
  g_signal_connect ((gpointer) size, "activate",
		    G_CALLBACK (image_zoom_set), (void *) SIZE_50);

  size = gtk_menu_item_new_with_mnemonic ("75%");
  gtk_object_set_data (GTK_OBJECT (size), "data", data);
  gtk_widget_show (size);
  gtk_container_add (GTK_CONTAINER (menu), size);
  g_signal_connect ((gpointer) size, "activate",
		    G_CALLBACK (image_zoom_set), (void *) SIZE_75);

  size = gtk_menu_item_new_with_mnemonic ("100%");
  gtk_object_set_data (GTK_OBJECT (size), "data", data);
  gtk_widget_show (size);
  gtk_container_add (GTK_CONTAINER (menu), size);
  g_signal_connect ((gpointer) size, "activate",
		    G_CALLBACK (image_zoom_set), (void *) SIZE_100);


  size = gtk_menu_item_new_with_mnemonic ("125%");
  gtk_object_set_data (GTK_OBJECT (size), "data", data);
  gtk_widget_show (size);
  gtk_container_add (GTK_CONTAINER (menu), size);
  g_signal_connect ((gpointer) size, "activate",
		    G_CALLBACK (image_zoom_set), (void *) SIZE_125);

  size = gtk_menu_item_new_with_mnemonic ("150%");
  gtk_object_set_data (GTK_OBJECT (size), "data", data);
  gtk_widget_show (size);
  gtk_container_add (GTK_CONTAINER (menu), size);
  g_signal_connect ((gpointer) size, "activate",
		    G_CALLBACK (image_zoom_set), (void *) SIZE_150);

  size = gtk_menu_item_new_with_mnemonic ("200%");
  gtk_object_set_data (GTK_OBJECT (size), "data", data);
  gtk_widget_show (size);
  gtk_container_add (GTK_CONTAINER (menu), size);
  g_signal_connect ((gpointer) size, "activate",
		    G_CALLBACK (image_zoom_set), (void *) SIZE_200);

  size = gtk_menu_item_new_with_mnemonic ("400%");
  gtk_object_set_data (GTK_OBJECT (size), "data", data);
  gtk_widget_show (size);
  gtk_container_add (GTK_CONTAINER (menu), size);
  g_signal_connect ((gpointer) size, "activate",
		    G_CALLBACK (image_zoom_set), (void *) SIZE_400);

  size = gtk_menu_item_new ();
  gtk_widget_show (size);
  gtk_container_add (GTK_CONTAINER (menu), size);

  size = gtk_menu_item_new_with_mnemonic ("window size");
  gtk_object_set_data (GTK_OBJECT (size), "data", data);
  gtk_widget_show (size);
  gtk_container_add (GTK_CONTAINER (menu), size);
  g_signal_connect ((gpointer) size, "activate",
		    G_CALLBACK (image_zoom_set), (void *) SIZE_AUTO);

  size = gtk_menu_item_new_with_mnemonic ("window width");
  gtk_object_set_data (GTK_OBJECT (size), "data", data);
  gtk_widget_show (size);
  gtk_container_add (GTK_CONTAINER (menu), size);
  g_signal_connect ((gpointer) size, "activate",
		    G_CALLBACK (image_zoom_set), (void *) SIZE_WIDTH);

  size = gtk_menu_item_new_with_mnemonic ("window height");
  gtk_object_set_data (GTK_OBJECT (size), "data", data);
  gtk_widget_show (size);
  gtk_container_add (GTK_CONTAINER (menu), size);
  g_signal_connect ((gpointer) size, "activate",
		    G_CALLBACK (image_zoom_set), (void *) SIZE_HEIGHT);

  return menu;
}

static gboolean
image_zoom_popup_show (GtkWidget * widget, GdkEventButton * event,
		       gpointer menu)
{
  DEBUG ("image zoom popup show");

  if (event->button == 3)
    {
      gtk_menu_popup (GTK_MENU (widget), NULL, NULL, NULL, NULL,
		      event->button, event->time);
      return TRUE;
    }

  return FALSE;
}

static GtkWidget *
image_create_image (GtkWidget * vp, char *file, gint size)
{
  GtkWidget *image;
  GdkPixbuf *i;
  gint x, y;

  DEBUG ("image create image");

  switch (size)
    {
    case SIZE_AUTO:
      x = vp->allocation.width;
      y = vp->allocation.height;

      i = gdk_pixbuf_new_from_file_at_size (file, x - 50, y - 50, NULL);
      break;

    case SIZE_HEIGHT:
      x = vp->allocation.width;
      y = vp->allocation.height;

      i = gdk_pixbuf_new_from_file (file, NULL);
      x = gdk_pixbuf_get_width (i);
      g_object_unref (i);

      i = gdk_pixbuf_new_from_file_at_size (file, x, y - 50, NULL);
      break;

    case SIZE_WIDTH:
      x = vp->allocation.width;
      y = vp->allocation.height;

      i = gdk_pixbuf_new_from_file (file, NULL);
      y = gdk_pixbuf_get_width (i);
      g_object_unref (i);

      i = gdk_pixbuf_new_from_file_at_size (file, x - 50, y, NULL);
      break;

    case SIZE_100:
      i = gdk_pixbuf_new_from_file (file, NULL);

      break;

    case SIZE_75:
      i = gdk_pixbuf_new_from_file (file, NULL);
      x = gdk_pixbuf_get_width (i);
      y = gdk_pixbuf_get_height (i);
      g_object_unref (i);

      x *= 75;
      x /= 100;

      y *= 75;
      y /= 100;

      i = gdk_pixbuf_new_from_file_at_size (file, x, y, NULL);
      break;

    case SIZE_50:
      i = gdk_pixbuf_new_from_file (file, NULL);
      x = gdk_pixbuf_get_width (i);
      y = gdk_pixbuf_get_height (i);
      g_object_unref (i);

      x *= 50;
      x /= 100;

      y *= 50;
      y /= 100;

      i = gdk_pixbuf_new_from_file_at_size (file, x, y, NULL);
      break;

    case SIZE_25:
      i = gdk_pixbuf_new_from_file (file, NULL);
      x = gdk_pixbuf_get_width (i);
      y = gdk_pixbuf_get_height (i);
      g_object_unref (i);

      x *= 25;
      x /= 100;

      y *= 25;
      y /= 100;

      i = gdk_pixbuf_new_from_file_at_size (file, x, y, NULL);
      break;

    case SIZE_125:
      i = gdk_pixbuf_new_from_file (file, NULL);
      x = gdk_pixbuf_get_width (i);
      y = gdk_pixbuf_get_height (i);
      g_object_unref (i);

      x *= 125;
      x /= 100;

      y *= 125;
      y /= 100;

      i = gdk_pixbuf_new_from_file_at_size (file, x, y, NULL);
      break;

    case SIZE_150:
      i = gdk_pixbuf_new_from_file (file, NULL);
      x = gdk_pixbuf_get_width (i);
      y = gdk_pixbuf_get_height (i);
      g_object_unref (i);

      x *= 150;
      x /= 100;

      y *= 150;
      y /= 100;

      i = gdk_pixbuf_new_from_file_at_size (file, x, y, NULL);
      break;

    case SIZE_200:
      i = gdk_pixbuf_new_from_file (file, NULL);
      x = gdk_pixbuf_get_width (i);
      y = gdk_pixbuf_get_height (i);
      g_object_unref (i);

      x *= 200;
      x /= 100;

      y *= 200;
      y /= 100;

      i = gdk_pixbuf_new_from_file_at_size (file, x, y, NULL);
      break;

    case SIZE_400:
      i = gdk_pixbuf_new_from_file (file, NULL);
      x = gdk_pixbuf_get_width (i);
      y = gdk_pixbuf_get_height (i);
      g_object_unref (i);

      x *= 400;
      x /= 100;

      y *= 400;
      y /= 100;

      i = gdk_pixbuf_new_from_file_at_size (file, x, y, NULL);
      break;


    default:
      i = gdk_pixbuf_new_from_file (file, NULL);
    }

  if (!i)
    {
      g_object_unref (i);
      image = gtk_label_new ("Generating image error!");

      return image;
    }

  image = gtk_image_new_from_pixbuf (i);
  if (!image)
    {
      g_object_unref (i);
      image = gtk_label_new ("Generating image error!");

      return image;
    }


  g_object_unref (i);

  return image;
}

static void
image_zoom_set (GtkWidget * w, int size)
{
  struct renderix_dialog_data *data;

  DEBUG ("image zoom set");

  data = gtk_object_get_data (GTK_OBJECT (w), "data");

  data->size = size;
  gtk_dialog_response (GTK_DIALOG (data->dialog), GTK_RESPONSE_CANCEL);
}

/* EOF */
