/* SomaX - Copyright (C) 2005 bakunin - Andrea Marchesini 
 *                                <bakunin@autistici.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#else
# error Use configure; make; make install
#endif

#include "renderix.h"
#include "plugins/plugins.h"

char **env;
GtkWidget *notebook;
GList *session = NULL;

void menu_session_close (GtkWidget *, gpointer);

gboolean
quit (GtkWidget * w, void *e, gpointer dummy)
{
  GList *tmp;
  struct renderix_data *data;
  int ok = 0;

  DEBUG ("quit");

  tmp = session;
  while (tmp)
    {
      data = tmp->data;
      if (data->proc)
	ok = 1;
      tmp = tmp->next;
    }

  if (ok
      && dialog_ask ("Working progress... You are sure to quit?") ==
      GTK_RESPONSE_OK)
    {
      tmp = session;
      while (tmp)
	{
	  session_stop (NULL, tmp->data);
	  tmp = tmp->next;
	}

      if (GTK_IS_WIDGET (w))
	gtk_widget_hide (gtk_widget_get_toplevel (w));

      gtk_main_quit ();
      return 1;

    }
  else if (!ok)
    {
      if (GTK_IS_WIDGET (w))
	gtk_widget_hide (gtk_widget_get_toplevel (w));

      gtk_main_quit ();
      return 1;

    }
  else
    return 0;

}

static GtkWidget *
graphic (void)
{
  GtkWidget *window;
  GtkWidget *box;
  GtkWidget *menubar;
  GtkWidget *menuitem;
  GtkWidget *menumenu;
  GtkWidget *plugin;
  GtkAccelGroup *accel;

  gchar s[1024];

  DEBUG ("graphic");

  accel = gtk_accel_group_new ();

  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  snprintf (s, sizeof (s), "%s %s", PACKAGE, VERSION);
  gtk_window_set_title (GTK_WINDOW (window), s);
  gtk_window_set_resizable (GTK_WINDOW (window), FALSE);

  g_signal_connect ((gpointer) window, "delete_event",
		    G_CALLBACK (quit), NULL);

  box = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (box);
  gtk_container_add (GTK_CONTAINER (window), box);

  menubar = gtk_menu_bar_new ();
  gtk_widget_show (menubar);
  gtk_box_pack_start (GTK_BOX (box), menubar, FALSE, FALSE, 0);

  menuitem = gtk_menu_item_new_with_mnemonic ("_File");
  gtk_widget_show (menuitem);
  gtk_container_add (GTK_CONTAINER (menubar), menuitem);

  menumenu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem), menumenu);

  menuitem = gtk_menu_item_new_with_mnemonic ("_New session");
  gtk_widget_show (menuitem);
  gtk_container_add (GTK_CONTAINER (menumenu), menuitem);

  if (!(plugin = plugin_new (session_new)))
    {
      dialog ("No plugin founded!\nRenderix can't run! Bye!");
      return NULL;
    }

  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem), plugin);

  menuitem = gtk_image_menu_item_new_from_stock ("gtk-close", accel);
  gtk_widget_show (menuitem);
  gtk_container_add (GTK_CONTAINER (menumenu), menuitem);
  g_signal_connect ((gpointer) menuitem, "activate",
		    GTK_SIGNAL_FUNC (menu_session_close), NULL);

  menuitem = gtk_menu_item_new ();
  gtk_widget_show (menuitem);
  gtk_container_add (GTK_CONTAINER (menumenu), menuitem);
  gtk_widget_set_sensitive (menuitem, FALSE);

  menuitem = gtk_image_menu_item_new_from_stock ("gtk-quit", accel);
  gtk_widget_show (menuitem);
  gtk_container_add (GTK_CONTAINER (menumenu), menuitem);
  g_signal_connect ((gpointer) menuitem, "activate",
		    GTK_SIGNAL_FUNC (quit), NULL);

  if (openmosix_is_up ())
    {
      menuitem = gtk_menu_item_new_with_mnemonic ("_OpenMosix");
      gtk_widget_show (menuitem);
      gtk_container_add (GTK_CONTAINER (menubar), menuitem);

      menumenu = gtk_menu_new ();
      gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem), menumenu);

      menuitem = gtk_menu_item_new_with_mnemonic ("OpenMosix _Stats");
      gtk_widget_show (menuitem);
      gtk_container_add (GTK_CONTAINER (menumenu), menuitem);
      g_signal_connect ((gpointer) menuitem, "activate",
			GTK_SIGNAL_FUNC (openmosix_stats_show), NULL);

      menuitem = gtk_menu_item_new_with_mnemonic ("OpenMosix _Graph");
      gtk_widget_show (menuitem);
      gtk_container_add (GTK_CONTAINER (menumenu), menuitem);
      g_signal_connect ((gpointer) menuitem, "activate",
			GTK_SIGNAL_FUNC (openmosix_graph_show), NULL);
    }

  menuitem = gtk_menu_item_new_with_mnemonic ("_Help");
  gtk_menu_item_set_right_justified (GTK_MENU_ITEM (menuitem), TRUE);
  gtk_widget_show (menuitem);
  gtk_container_add (GTK_CONTAINER (menubar), menuitem);

  menumenu = gtk_menu_new ();
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem), menumenu);

  menuitem = gtk_image_menu_item_new_from_stock ("gtk-about", accel);
  gtk_widget_show (menuitem);
  gtk_container_add (GTK_CONTAINER (menumenu), menuitem);
  g_signal_connect ((gpointer) menuitem, "activate",
		    GTK_SIGNAL_FUNC (about_show), NULL);

  notebook = gtk_notebook_new ();
  gtk_widget_show (notebook);
  gtk_box_pack_start (GTK_BOX (box), notebook, TRUE, TRUE, 0);
  gtk_notebook_set_scrollable (GTK_NOTEBOOK (notebook), TRUE);

  gtk_window_add_accel_group (GTK_WINDOW (window), accel);

  return window;
}

int
main (int argc, char **argv, char **e)
{
  GtkWidget *window;
  pthread_t th;

  env = e;

  signal(SIGPIPE, signal_pipe);
  signal (SIGINT, signal_handler);

  gtk_init (NULL, NULL);

  plugin_set_env (env);
  plugin_scanner ();

  timer_init ();

  if (openmosix_is_up ())
    {
      pthread_create (&th, NULL, openmosix_thread, NULL);
      openmosix_init ();
      openmosix_stats ();
      openmosix_graph ();
    }

  about_init ();

  if (!(window = graphic ()))
    return 1;

  session_new (NULL, NULL);

  gtk_widget_show (window);

  if (!openmosix_is_up ())
    dialog ("No openmosix detected!\nRenderix runs without cluster support!");

  gtk_main ();

  if (openmosix_is_up ())
    {
      openmosix_quit ();
      pthread_join (th, NULL);
    }

  timer_destroy ();

  return 0;
}

void
menu_session_close (GtkWidget * w, gpointer dummy)
{
  gint page;
  GList *i;

  DEBUG ("menu session close");

  page = gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook));

  i = session;
  while (i)
    {
      if (!page)
	{
	  session_close (NULL, i->data);
	  break;
	}

      page--;
      i = i->next;
    }
}

/* EOF */
