/* SomaX - Copyright (C) 2005 bakunin - Andrea Marchesini 
 *                                <bakunin@autistici.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#else
# error Use configure; make; make install
#endif

#include "../renderix.h"

#define LABEL_NEW( x ) label = gtk_label_new (x); \
  gtk_widget_show (label); \
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, i, i+1, \
                    (GtkAttachOptions) (GTK_FILL | GTK_EXPAND), \
                    (GtkAttachOptions) (0), 0, 0); \
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

#define DATA_NEW( x ) gtk_widget_show (x); \
  gtk_table_attach (GTK_TABLE (table), x, 1, 2, i, i+1, \
                    (GtkAttachOptions) (GTK_FILL | GTK_EXPAND), \
                    (GtkAttachOptions) (0), 0, 0);

static GtkWidget *blender_start;
static GtkWidget *blender_stop;
static GtkWidget *blender_nodes;
static GtkWidget *blender_file;

static void entry_add (GtkWidget *, GtkWidget *);

char *
module_name (void)
{
  DEBUG("blender: module_name");
  return "blender";
}

char *
module_author (void)
{
  DEBUG("blender: module_author");
  return "Andrea Marchesini";
}

char *
module_licence (void)
{
  DEBUG("blender: module_licence");
  return "GPL";
}

char *
module_description (void)
{
  DEBUG("blender: module_description");
  return "blender rendering";
}

char *
module_version (void)
{
  DEBUG("blender: module_version");
  return "0.1";
}

GtkWidget *
module_widget (void)
{
  GtkWidget *frame;
  GtkWidget *table;
  GtkWidget *label;
  GtkWidget *entry;
  GtkWidget *button;
  GtkWidget *box;
  gint i = 0;

  char s[1024];

  DEBUG("blender: module_widget");

  frame = gtk_frame_new ("Render Config");
  gtk_widget_show (frame);

  table = gtk_table_new (0, 0, FALSE);
  gtk_widget_show (table);
  gtk_container_add (GTK_CONTAINER (frame), table);

  LABEL_NEW ("File:");

  box = gtk_hbox_new (FALSE, 0);
  DATA_NEW (box);
  i++;

  entry = gtk_entry_new ();
  blender_file = entry;
  gtk_widget_show (entry);
  gtk_box_pack_start (GTK_BOX (box), entry, TRUE, TRUE, 0);

  button = gtk_button_new_with_label ("Search");
  gtk_widget_show (button);
  gtk_box_pack_start (GTK_BOX (box), button, TRUE, TRUE, 0);
  g_signal_connect ((gpointer) button, "clicked",
		    G_CALLBACK (entry_add), entry);

  LABEL_NEW ("N. Nodes:");

  entry = gtk_entry_new ();
  if (openmosix_is_up ())
    {
      snprintf (s, sizeof (s), "%d", openmosix_n_nodes ());
      gtk_entry_set_text (GTK_ENTRY (entry), s);
    }

  blender_nodes = entry;
  DATA_NEW (entry);
  i++;

  LABEL_NEW ("Start Frame:");

  entry = gtk_entry_new ();
  blender_start = entry;
  DATA_NEW (entry);
  i++;

  LABEL_NEW ("Stop Frame:");

  entry = gtk_entry_new ();
  blender_stop = entry;
  DATA_NEW (entry);
  i++;

  return frame;
}

static void
entry_add (GtkWidget * w, GtkWidget * entry)
{
  char *file;

  DEBUG ("entry add");

  if (!
      (file =
       file_chooser ("Search your file", GTK_SELECTION_SINGLE,
		     GTK_FILE_CHOOSER_ACTION_OPEN)))
    return;

  gtk_entry_set_text (GTK_ENTRY (entry), file);
  free (file);
}

GList *
module_data (void)
{
  int i;
  char s[1024];
  struct renderix_item_data *data;
  struct renderix_item *item;
  GList *list=NULL;
  char **arg;
  gint p_start;
  gint p_stop;

  DEBUG ("blender: module_command_list");

  gchar *file = (gchar *) gtk_entry_get_text (GTK_ENTRY (blender_file));
  gint nodes = atoi ((char *) gtk_entry_get_text (GTK_ENTRY (blender_nodes)));
  gint start = atoi ((char *) gtk_entry_get_text (GTK_ENTRY (blender_start)));
  gint stop = atoi ((char *) gtk_entry_get_text (GTK_ENTRY (blender_stop)));
  gint mod;
  gint x;

  gint images;

  if (!start)
    {
      start++;
      snprintf (s, sizeof (s), "%d", start);
      gtk_entry_set_text (GTK_ENTRY (blender_start), s);
    }

  if (!stop)
    {
      stop++;
      snprintf (s, sizeof (s), "%d", stop);
      gtk_entry_set_text (GTK_ENTRY (blender_stop), s);
    }

  if (!nodes)
    {
      nodes++;
      snprintf (s, sizeof (s), "%d", nodes);
      gtk_entry_set_text (GTK_ENTRY (blender_nodes), s);
    }

  if (start > stop)
    {
      dialog ("No correct data!");
      return NULL;
    }

  if (nodes > 5)
    {
      if (!openmosix_is_up ()
	  &&
	  dialog_ask
	  ("No openmosix detected!\nWith too many nodes, your system will be blocked\nContinue?")
	  != GTK_RESPONSE_OK)
	return NULL;
    }

  images = ((stop + 1) - start) / nodes;
  mod = ((stop + 1) - start) % nodes;

  start--;
  stop--;

  x = start;

  for (i = 0; i < nodes; i++)
    {

      item = g_malloc(sizeof(struct renderix_item));
      memset(item, 0, sizeof(struct renderix_item));

      p_start = x;
      p_stop = x + (images - 1);

      data = g_malloc(sizeof(struct renderix_item_data));
      memset(data, 0, sizeof(struct renderix_item_data));
      item->data=g_list_append(item->data, data);
      
      data->text=g_strdup("Frame Start:");
      snprintf(s, sizeof(s), "%d",p_start+1);
      data->value=g_strdup(s);

      data = g_malloc(sizeof(struct renderix_item_data));
      memset(data, 0, sizeof(struct renderix_item_data));
      item->data=g_list_append(item->data, data);
      
      data->text=g_strdup("Frame Stop:");
      snprintf(s, sizeof(s), "%d",p_stop+1);
      data->value=g_strdup(s);

      if (mod)
	{
	  p_stop++;
	  mod--;
	}

      x = p_stop + 1;

      arg = (char **) g_malloc (sizeof (char *) * 10);

      arg[0] = g_strdup (BLENDER);
      arg[1] = g_strdup ("-b");
      arg[2] = g_strdup (file);
      arg[3] = g_strdup ("-s");

      snprintf (s, sizeof (s), "%d", p_start);
      arg[4] = g_strdup (s);

      arg[5] = g_strdup ("-e");

      snprintf (s, sizeof (s), "%d", p_stop);
      arg[6] = g_strdup (s);

      arg[7] = g_strdup ("-a");
      arg[8] = NULL;

      item->arg=arg;
      list = g_list_append(list, item);
    }

#ifdef ENABLE_DEBUG
  snprintf(s, sizeof(s), "blender: %d commands.",g_list_length(list));
  DEBUG(s);
#endif

  return list;
}

/* EOF */
