/* SomaX - Copyright (C) 2005 bakunin - Andrea Marchesini 
 *                                <bakunin@autistici.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#else
# error Use configure; make; make install
#endif

#include "renderix.h"

struct openmosix_data
{
  gint id;
  gboolean speed;
  gint load;
  gint cpus;
  gint status;
  gint util;
  gint mem;
  gint rmem;
  gint tmem;
};

static GList *op_list = NULL;
static pthread_mutex_t op_mutex;
static gboolean op_quit = FALSE;
static GtkWidget *graph_window;
static GtkWidget *stats_window;

static gboolean openmosix_refresh_stats (GtkWidget *);
static gboolean openmosix_stats_close (GtkWidget *, gpointer);
static gboolean openmosix_refresh_graph (GtkWidget *);
static gboolean openmosix_graph_close (GtkWidget *, gpointer);

void
openmosix_init (void)
{
  DEBUG ("Openmosix init");
  pthread_mutex_init (&op_mutex, NULL);
}

gboolean
openmosix_is_up (void)
{
  DEBUG ("Openmosix is up");
  return msx_is_mosix ();
}

gint
openmosix_n_nodes (void)
{
  DIR *d;
  struct dirent *dir;
  gint i = 0;
  gint node;
  gint ret;

  DEBUG ("Openmosix n_nodes");
  if (!(d = opendir ("/proc/hpc/nodes")))
    return 0;

  while ((dir = readdir (d)))
    if (*dir->d_name != '.')
      {
	node = atoi (dir->d_name);
	ret = msx_readnode (node, "cpus");
	i += ret > 0 ? ret : 0;
      }

  closedir (d);

  return i;
}

void
openmosix_stats_show (void)
{
  DEBUG ("Openmosix stats show");
  if (GTK_WIDGET_VISIBLE (stats_window))
    gtk_widget_hide (stats_window);
  else
    gtk_widget_show_all (stats_window);
}

void
openmosix_graph_show (void)
{
  DEBUG ("Openmosix graph show");
  if (GTK_WIDGET_VISIBLE (graph_window))
    gtk_widget_hide (graph_window);
  else
    gtk_widget_show_all (graph_window);
}

void
openmosix_stats (void)
{
  GtkWidget *window;
  GtkWidget *button;
  GtkWidget *box;
  GtkWidget *hbox;
  GtkWidget *sep;
  GtkWidget *sw;
  GtkWidget *tv;
  GtkListStore *model;
  GtkTreeSelection *selection;
  GtkCellRenderer *renderer;

  char s[1024];

  DEBUG ("Openmosix stats");

  snprintf (s, sizeof (s), "%s %s - OpenMosix Stats", PACKAGE, VERSION);

  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (window), s);
  gtk_window_set_modal (GTK_WINDOW (window), FALSE);

  box = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (window), box);

  sw = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_set_size_request (sw, 400, 500);
  gtk_box_pack_start (GTK_BOX (box), sw, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
				  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  model = gtk_list_store_new (9, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT,
			      G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING,
			      G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);

  tv = gtk_tree_view_new_with_model (GTK_TREE_MODEL (model));
  gtk_tree_view_set_enable_search (GTK_TREE_VIEW (tv), FALSE);
  gtk_container_add (GTK_CONTAINER (sw), tv);
  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tv));
  gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE);
  gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (tv), TRUE);
  g_object_unref (model);

  renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (tv),
					       -1, "ID", renderer,
					       "text", 0, NULL);

  renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (tv),
					       -1, "Running", renderer,
					       "text", 1, NULL);

  renderer = gtk_cell_renderer_progress_new ();
  gtk_cell_renderer_set_fixed_size (renderer, 100, -1);
  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (tv),
					       -1, "Load", renderer,
					       "value", 2, NULL);

  renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (tv),
					       -1, "N. Cpu", renderer,
					       "text", 3, NULL);

  renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (tv),
					       -1, "Status", renderer,
					       "text", 4, NULL);

  renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (tv),
					       -1, "Util", renderer,
					       "text", 5, NULL);

  renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (tv),
					       -1, "Mem", renderer,
					       "text", 6, NULL);

  renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (tv),
					       -1, "R. Mem", renderer,
					       "text", 7, NULL);

  renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (tv),
					       -1, "T. Mem", renderer,
					       "text", 8, NULL);

  sep = gtk_hseparator_new ();
  gtk_box_pack_start (GTK_BOX (box), sep, FALSE, FALSE, 0);
  gtk_widget_set_size_request (sep, -1, 10);

  hbox = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (box), hbox, FALSE, FALSE, 0);

  button = gtk_label_new ("");
  gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);

  button = gtk_button_new_from_stock ("gtk-ok");
  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);

  openmosix_refresh_stats (tv);
  g_timeout_add (1000, (GSourceFunc) openmosix_refresh_stats, tv);

  g_signal_connect (window, "delete_event",
		    G_CALLBACK (openmosix_stats_close), NULL);
  g_signal_connect (button, "clicked", G_CALLBACK (openmosix_stats_close),
		    NULL);

  stats_window = window;
}

void
openmosix_graph (void)
{
  GtkWidget *window;
  GtkWidget *button;
  GtkWidget *graphic;
  GtkWidget *box;
  GtkWidget *hbox;
  GtkWidget *sep;

  char s[1024];

  DEBUG ("Openmosix graph");

  snprintf (s, sizeof (s), "%s %s - OpenMosix Graph", PACKAGE, VERSION);

  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (window), s);
  gtk_window_set_modal (GTK_WINDOW (window), FALSE);

  box = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (window), box);

  graphic = graphic_new ();
  gtk_box_pack_start (GTK_BOX (box), graphic, TRUE, TRUE, 0);
  gtk_widget_set_size_request (graphic, 600, 400);
  graphic_set_range (graphic, 60);

  sep = gtk_hseparator_new ();
  gtk_widget_set_size_request (sep, -1, 10);
  gtk_box_pack_start (GTK_BOX (box), sep, FALSE, FALSE, 0);

  hbox = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (box), hbox, FALSE, FALSE, 0);

  button = gtk_label_new ("");
  gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);

  button = gtk_button_new_from_stock ("gtk-ok");
  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);

  openmosix_refresh_graph (graphic);
  g_timeout_add (1000, (GSourceFunc) openmosix_refresh_graph, graphic);

  g_signal_connect (window, "delete_event",
		    G_CALLBACK (openmosix_graph_close), NULL);
  g_signal_connect (button, "clicked", G_CALLBACK (openmosix_graph_close),
		    NULL);

  graph_window = window;
}

static gboolean
openmosix_stats_close (GtkWidget * w, gpointer d)
{
  DEBUG ("Openmosix stats close");

  gtk_widget_hide (gtk_widget_get_toplevel (w));
  return TRUE;
}

static gboolean
openmosix_graph_close (GtkWidget * w, gpointer d)
{
  DEBUG ("Openmosix graph close");

  gtk_widget_hide (gtk_widget_get_toplevel (w));
  return TRUE;
}

void *
openmosix_thread (void *dummy)
{
  DIR *d;
  struct dirent *dir;
  int node;
  struct openmosix_data *data;
  GList *tmp;

  DEBUG ("Openmosix thread");

  while (op_quit == FALSE)
    {
      pthread_mutex_lock (&op_mutex);

      DEBUG ("Openmosix thread - free list");

      if (op_list)
	{
	  tmp = op_list;

	  while (tmp)
	    {
	      free (tmp->data);
	      tmp = tmp->next;
	    }

	  g_list_free (op_list);
	  op_list = NULL;
	}

      DEBUG ("Openmosix thread - read directory");

      if (!(d = opendir ("/proc/hpc/nodes")))
	{
	  pthread_exit (NULL);
	  return NULL;
	}

      while ((dir = readdir (d)))
	{
	  if (*dir->d_name != '.')
	    {
	      DEBUG ("Openmosix thread - new data");

	      if (op_quit == TRUE)
		break;

	      node = atoi (dir->d_name);

	      data = g_malloc (sizeof (struct openmosix_data));
	      op_list = g_list_append (op_list, data);

	      data->id = node;
	      data->speed = msx_readnode (node, "speed") > 0 ? TRUE : FALSE;

	      data->load = msx_readnode (node, "load");
	      if (data->load > 0)
		{
		  if (data->load > 200)
		    data->load = 100;
		  else
		    data->load = data->load / 2;
		}
	      else
		data->load = 0;

	      data->cpus = msx_readnode (node, "cpus");
	      data->status = msx_readnode (node, "status");
	      data->util = msx_readnode (node, "util");
	      data->mem = msx_readnode (node, "mem") / 1048576;
	      data->rmem = msx_readnode (node, "rmem") / 1048576;
	      data->tmem = msx_readnode (node, "tmem") / 1048576;
	    }
	}

      closedir (d);

      pthread_mutex_unlock (&op_mutex);

      if (op_quit == TRUE)
	break;

      DEBUG ("Openmosix thread - sleep");
      sleep (2);
    }

  DEBUG ("Openmosix thread - quit");
  pthread_exit (NULL);
  return NULL;
}

static gboolean
openmosix_refresh_stats (GtkWidget * tv)
{
  struct openmosix_data *data;
  GtkListStore *model;
  GtkTreeIter iter;
  GList *tmp;
  char s[1024];

  DEBUG ("Openmosix stats refresh");

  if (pthread_mutex_trylock (&op_mutex) != EBUSY)
    {

      DEBUG ("Openmosix stats locked");

      model = (GtkListStore *) gtk_tree_view_get_model (GTK_TREE_VIEW (tv));

      while (gtk_tree_model_iter_nth_child
	     (GTK_TREE_MODEL (model), &iter, NULL, 0))
	gtk_list_store_remove (model, &iter);

      tmp = op_list;
      while (tmp)
	{
	  gtk_list_store_append (model, &iter);
	  data = tmp->data;

	  snprintf (s, sizeof (s), "%d", data->id);
	  gtk_list_store_set (model, &iter, 0, s, -1);

	  snprintf (s, sizeof (s), "%s", data->speed == TRUE ? "Yes" : "No");
	  gtk_list_store_set (model, &iter, 1, s, -1);

	  gtk_list_store_set (model, &iter, 2, data->load, -1);

	  snprintf (s, sizeof (s), "%d", data->cpus);
	  gtk_list_store_set (model, &iter, 3, s, -1);

	  snprintf (s, sizeof (s), "%d", data->status);
	  gtk_list_store_set (model, &iter, 4, s, -1);

	  snprintf (s, sizeof (s), "%d", data->util);
	  gtk_list_store_set (model, &iter, 5, s, -1);

	  snprintf (s, sizeof (s), "%d", data->mem);
	  gtk_list_store_set (model, &iter, 6, s, -1);

	  snprintf (s, sizeof (s), "%d", data->rmem);
	  gtk_list_store_set (model, &iter, 7, s, -1);

	  snprintf (s, sizeof (s), "%d", data->tmem);
	  gtk_list_store_set (model, &iter, 8, s, -1);

	  tmp = tmp->next;
	}

      DEBUG ("Openmosix stats unlocked");

      pthread_mutex_unlock (&op_mutex);

    }

  DEBUG ("Openmosix stats refresh stop");

  return TRUE;
}

static gboolean
openmosix_refresh_graph (GtkWidget * graphic)
{
  struct openmosix_data *data;
  GList *tmp;
  static gint total = 0;
  static gint n = 0;

  DEBUG ("Openmosix graph refresh");

  if (pthread_mutex_trylock (&op_mutex) != EBUSY)
    {

      DEBUG ("Openmosix graph locked");

      total = 0;
      tmp = op_list;
      while (tmp)
	{
	  data = tmp->data;

	  total += data->load;

	  tmp = tmp->next;
	}

      n = g_list_length (op_list);

      DEBUG ("Openmosix graph unlocked");

      pthread_mutex_unlock (&op_mutex);

      if (n)
	{
	  total /= n;
	  graphic_set_data (graphic, total);
	}

      else
	graphic_set_data (graphic, 0);
    }
  else
    graphic_set_data (graphic, total);

  DEBUG ("Openmosix graph refresh stop");

  return TRUE;
}

void
openmosix_quit (void)
{
  DEBUG ("Openmosix quit");

  op_quit = TRUE;
}

/* EOF */
