/* SomaX - Copyright (C) 2005 bakunin - Andrea Marchesini 
 *                                <bakunin@autistici.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#else
# error Use configure; make; make install
#endif

#include "plugins.h"

static char **plugin_env = NULL;
static GList *plugin_list = NULL;
static struct plugin_t *plugin_get_data (gchar *);
static GtkWidget *plugin_splash_init ();
static void plugin_splash_close (GtkWidget *);
static void plugin_splash_text (GtkWidget *, char *, ...);

void
plugin_scanner (void)
{
  DIR *d;
  struct dirent *dir;
  struct stat st;
  struct plugin_t *tmp;
  gchar *a;
  GtkWidget *w;

  w=plugin_splash_init();

  if (!(d = opendir (PLUGINS_PATH)))
    {
      plugin_splash_close (w);
      return;
    }

  while ((dir = readdir (d)) != NULL)
    {
      if (*dir->d_name == '.')
	continue;

      plugin_splash_text (w, "%s", dir->d_name);

      a =
	(gchar *) g_malloc (sizeof (gchar) *
			    (strlen (PLUGINS_PATH) + strlen (dir->d_name) +
			     3));
      sprintf (a, "%s/%s", PLUGINS_PATH, dir->d_name);

      if (lstat (a, &st) || !S_ISREG (st.st_mode))
	{
	  g_free (a);
	  continue;
	}

      if (!(tmp = plugin_get_data (a)))
	{
	  g_free (a);
	  continue;
	}

      tmp->file = a;
      plugin_list = g_list_append (plugin_list, tmp);
    }

  closedir (d);

  plugin_splash_close (w);
}

static void plugin_free_data(struct plugin_t *tmp) {
  if(!tmp)return;

  if(tmp->library) dlclose(tmp->library);

  free(tmp);
}

static struct plugin_t *
plugin_get_data (gchar * a)
{
  struct plugin_t *tmp;
  void *lib;
  char *(*func) (void);

  tmp = g_malloc (sizeof (struct plugin_t));
  memset (tmp, 0, sizeof (struct plugin_t));

  if (!(lib = dlopen (a, RTLD_LAZY)))
    {
      plugin_free_data(tmp);
      return NULL;
    }

  if ((func = dlsym (lib, "module_name")))
    {
      if (!(tmp->name = func ()))
	{
          fprintf(stderr,"No data in module_name function!\n");
          plugin_free_data(tmp);
	  return NULL;
	}
    }
  else
    {
      fprintf(stderr,"No module_name function!\n");
      plugin_free_data(tmp);
      return NULL;
    }

  if ((func = dlsym (lib, "module_author")))
    {
      if (!(tmp->author = func ()))
	{
          fprintf(stderr,"No data in module_author function!\n");
          plugin_free_data(tmp);
	  return NULL;
	}
    }
  else
    {
      fprintf(stderr,"No module_author function!\n");
      plugin_free_data(tmp);
      return NULL;
    }

  if ((func = dlsym (lib, "module_description")))
    {
      if (!(tmp->description = func ()))
	{
          fprintf(stderr,"No data in module_description function!\n");
          plugin_free_data(tmp);
	  return NULL;
	}
    }
  else
    {
      fprintf(stderr,"No module_description function!\n");
      plugin_free_data(tmp);
      return NULL;
    }

  if ((func = dlsym (lib, "module_version")))
    {
      if (!(tmp->version = func ()))
	{
          fprintf(stderr,"No data in module_version function!\n");
          plugin_free_data(tmp);
	  return NULL;
	}
    }
  else
    {
      fprintf(stderr,"No module_version function!\n");
      plugin_free_data(tmp);
      return NULL;
    }

  if ((func = dlsym (lib, "module_licence")))
    {
      if (!(tmp->licence = func ()))
	{
          fprintf(stderr,"No data in module_licence function!\n");
          plugin_free_data(tmp);
	  return NULL;
	}
    }
  else
    {
      fprintf(stderr,"No module_licence function!\n");
      plugin_free_data(tmp);
      return NULL;
    }

  if (!(tmp->widget = dlsym (lib, "module_widget")))
    {
      fprintf(stderr,"No module_widget function!\n");
      plugin_free_data(tmp);
      return NULL;
    }

  if (!(tmp->data = dlsym (lib, "module_data")))
    {
      fprintf(stderr,"No module_data function!\n");
      plugin_free_data(tmp);
      return NULL;
    }

  tmp->library=lib;
  tmp->file=g_strdup(a);
  tmp->name=g_strdup(tmp->name);
  tmp->author=g_strdup(tmp->author);
  tmp->description=g_strdup(tmp->description);
  tmp->version=g_strdup(tmp->version);
  tmp->licence=g_strdup(tmp->licence);

  return tmp;
}

void
plugin_starter (struct plugin_t *data, char *file)
{
  /* DA FARE */
}

static GtkWidget *
plugin_splash_init ()
{
  GtkWidget *window;
  GtkWidget *vbox;
  GtkWidget *image;
  GdkPixbuf *i;
  GtkWidget *label;

  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (window), PACKAGE);
  gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_CENTER);
  gtk_window_set_resizable (GTK_WINDOW (window), FALSE);
  gtk_widget_realize (window);

  vbox = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox);
  gtk_container_add (GTK_CONTAINER (window), vbox);

  i = gdk_pixbuf_new_from_file (PATH_IMAGE, NULL);
  image = gtk_image_new_from_pixbuf (i);
  gtk_widget_show (image);

  gtk_box_pack_start (GTK_BOX (vbox), image, TRUE, TRUE, 0);

  label = gtk_label_new ("Test module:");
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_CENTER);

  label = gtk_label_new ("initialize data");
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_CENTER);

  gtk_object_set_data (GTK_OBJECT (window), "label", label);

  gtk_widget_show (window);

  while (gtk_events_pending ())
    gtk_main_iteration ();
  usleep (500000);

  return window;
}

/* The message on boot splash */
static void
plugin_splash_text (GtkWidget * w, char *str, ...)
{

  char buf[100];
  va_list va;
  GtkWidget *label;

  label = gtk_object_get_data (GTK_OBJECT (w), "label");

  va_start (va, str);

  vsnprintf (buf, 100, str, va);
  gtk_label_set_text (GTK_LABEL (label), buf);

  while (gtk_events_pending ())
    gtk_main_iteration ();

  va_end (va);
}

/* Close the bootsplash */
static void
plugin_splash_close (GtkWidget * w)
{
  gtk_widget_destroy (w);

  while (gtk_events_pending ())
    gtk_main_iteration ();
}

void
plugin_about (void)
{
  GtkWidget *dialog;
  GtkWidget *button;
  GtkWidget *sw;
  GtkWidget *tv;
  GtkListStore *model;
  GtkTreeIter iter;
  GtkTreeSelection *selection;
  GtkCellRenderer *renderer;
  GList *tmp;

  char s[1024];

  snprintf (s, sizeof (s), "%s %s - plugins about", PACKAGE, VERSION);

  dialog = gtk_dialog_new ();
  gtk_widget_set_size_request (dialog, 400, 300);
  gtk_window_set_title (GTK_WINDOW (dialog), s);
  gtk_window_set_type_hint (GTK_WINDOW (dialog), GDK_WINDOW_TYPE_HINT_DIALOG);
  gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);

  sw = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (sw);
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), sw, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
				  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  model =
    gtk_list_store_new (5, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING,
			G_TYPE_STRING, G_TYPE_STRING);
  tv = gtk_tree_view_new_with_model (GTK_TREE_MODEL (model));
  gtk_tree_view_set_enable_search (GTK_TREE_VIEW (tv), FALSE);
  gtk_container_add (GTK_CONTAINER (sw), tv);
  gtk_widget_show (tv);
  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tv));
  gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE);
  gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (tv), TRUE);
  g_object_unref (model);

  renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (tv),
					       -1, "Plugin", renderer,
					       "text", 0, NULL);

  renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (tv),
					       -1, "Author", renderer,
					       "text", 1, NULL);

  renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (tv),
					       -1, "Version", renderer,
					       "text", 2, NULL);

  renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (tv),
					       -1, "Licence", renderer,
					       "text", 3, NULL);

  renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (tv),
					       -1, "Description", renderer,
					       "text", 4, NULL);

  tmp = plugin_list;
  while (tmp)
    {
      gtk_list_store_append (model, &iter);
      gtk_list_store_set (model, &iter,
			  0, ((struct plugin_t *) tmp->data)->name,
			  1, ((struct plugin_t *) tmp->data)->author,
			  2, ((struct plugin_t *) tmp->data)->version,
			  3, ((struct plugin_t *) tmp->data)->licence,
			  4, ((struct plugin_t *) tmp->data)->description,
			  -1);
      tmp = tmp->next;
    }

  button = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (button);
  gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);

  gtk_widget_show_all (dialog);
  gtk_dialog_run (GTK_DIALOG (dialog));
  gtk_widget_destroy (dialog);
}

GtkWidget *
plugin_new (void *func)
{
  GList *tmp;
  GtkWidget *menu;
  GtkWidget *image;
  GtkWidget *item;

  if (!plugin_list)
    return NULL;

  tmp = plugin_list;

  menu = gtk_menu_new ();

  while (tmp)
    {
      item = gtk_menu_item_new_with_mnemonic (((struct plugin_t *)
					       tmp->data)->description);
      gtk_widget_show (item);
      gtk_container_add (GTK_CONTAINER (menu), item);
      g_signal_connect ((gpointer) item, "activate",
			GTK_SIGNAL_FUNC (func), tmp->data);
      tmp = tmp->next;
    }

  item = gtk_menu_item_new ();
  gtk_widget_show (item);
  gtk_container_add (GTK_CONTAINER (menu), item);
  gtk_widget_set_sensitive (item, FALSE);

  item = gtk_image_menu_item_new_with_mnemonic ("Plugins _About");
  gtk_widget_show (item);
  gtk_container_add (GTK_CONTAINER (menu), item);
  g_signal_connect ((gpointer) item, "activate",
		    GTK_SIGNAL_FUNC (plugin_about), NULL);

  image = gtk_image_new_from_stock ("gtk-about", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image);
  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);

  return menu;
}

void
plugin_set_env (char **env)
{
  plugin_env = env;
}

GList *plugin_get_list(void) {return plugin_list;}

/* EOF */
