/* SomaX - Copyright (C) 2005 bakunin - Andrea Marchesini 
 *                                <bakunin@autistici.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#else
# error Use configure; make; make install
#endif

#include "renderix.h"

static void *proc_exec (void *);
static void proc_free (struct renderix_data *);
static void proc_startstop (GtkWidget *, struct renderix_proc *);
static void proc_kill (GtkWidget *, struct renderix_proc *);
static void proc_killall (GtkWidget *, struct renderix_data *);
static void proc_pauseall (GtkWidget *, struct renderix_data *);

void
proc_new (struct renderix_proc *proc)
{
  DEBUG ("proc new");

  proc->time = timer_new ();
  proc->time_image = timer_new ();

  pthread_create (&proc->th, NULL, proc_exec, proc);
}

static void *
proc_exec (void *dummy)
{
  struct renderix_proc *proc = (struct renderix_proc *) dummy;
  int p[2];

  char buf;
  int k;

  DEBUG ("proc exec");

  pipe (p);

  switch ((proc->pid = fork ()))
    {
    case -1:
      proc->status = 2;
      pthread_exit (NULL);
      break;

    case 0:
      close (p[0]);
      close (STDOUT_FILENO);
      close (STDERR_FILENO);
      dup2 (p[1], STDOUT_FILENO);
      dup2 (p[1], STDERR_FILENO);

      execve (proc->arg[0], proc->arg, env);
      perror (proc->arg[0]);
      exit (1);
      break;

    default:
      close (p[1]);

      while (waitpid (proc->pid, NULL, WNOHANG) != proc->pid)
	{
	  pthread_mutex_lock (&proc->mutex);

	  if (*proc->buffer)
	    pthread_cond_wait (&proc->cond, &proc->mutex);

	  k = 0;

	  while (read (p[0], &buf, sizeof (char)) == 1)
	    {
	      if (buf == '\r' || buf == '\n'
		  || k == sizeof (proc->buffer) - 1)
		{
		  proc->buffer[k] = 0;
		  break;
		}
	      else
		proc->buffer[k++] = buf;
	    }

	  pthread_mutex_unlock (&proc->mutex);
	}

      proc->status = 1;
      pthread_exit (NULL);
      break;
    }

  return NULL;
}

void
proc_window (struct renderix_data *data)
{
  GtkWidget *notebook;
  GtkWidget *box;
  GtkWidget *gbox;
  GtkWidget *hbox;
  GtkWidget *label;
  GList *item;
  struct renderix_proc *proc;
  struct renderix_item_data *item_data;
  GList *item_list;

  int n = 0;
  char s[1024];

  DEBUG ("proc window");

  gbox = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (gbox);
  gtk_box_pack_start (GTK_BOX (data->body), gbox, FALSE, FALSE, 0);
  data->jobs = gbox;

  box = gtk_hbox_new (TRUE, 0);
  gtk_widget_show (box);
  gtk_box_pack_start (GTK_BOX (gbox), box, TRUE, TRUE, 0);

  label = gtk_button_new_with_mnemonic ("Pause all jobs");
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (box), label, TRUE, TRUE, 0);

  g_signal_connect ((gpointer) label, "clicked",
		    G_CALLBACK (proc_pauseall), data);

  label = gtk_button_new_with_mnemonic ("Kill all jobs");
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (box), label, TRUE, TRUE, 0);

  g_signal_connect ((gpointer) label, "clicked",
		    G_CALLBACK (proc_killall), data);

  notebook = gtk_notebook_new ();
  gtk_notebook_set_scrollable (GTK_NOTEBOOK (notebook), TRUE);
  gtk_notebook_set_tab_pos (GTK_NOTEBOOK (notebook), GTK_POS_LEFT);
  gtk_widget_show (notebook);
  gtk_box_pack_start (GTK_BOX (gbox), notebook, TRUE, TRUE, 0);

  item = data->proc;
  while (item)
    {
      proc = item->data;

      snprintf (s, sizeof (s), "Node %d", n);
      label = gtk_label_new (s);
      gtk_widget_show (label);

      box = gtk_vbox_new (FALSE, 0);
      gtk_widget_show (box);

      gtk_notebook_append_page (GTK_NOTEBOOK (notebook), box, label);

      hbox = gtk_hbox_new (FALSE, 0);
      gtk_widget_show (hbox);
      gtk_box_pack_start (GTK_BOX (box), hbox, FALSE, FALSE, 0);

      label = gtk_label_new ("Status:");
      gtk_widget_show (label);
      gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
      gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);

      label = gtk_label_new ("RUNNING");
      gtk_widget_show (label);
      gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
      proc->w_status = label;
      gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);

      hbox = gtk_hbox_new (FALSE, 0);
      gtk_widget_show (hbox);
      gtk_box_pack_start (GTK_BOX (box), hbox, FALSE, FALSE, 0);

      label = gtk_label_new ("Time:");
      gtk_widget_show (label);
      gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
      gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);

      label = gtk_label_new ("");
      gtk_widget_show (label);
      gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
      proc->w_time = label;
      gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);

      hbox = gtk_hbox_new (FALSE, 0);
      gtk_widget_show (hbox);
      gtk_box_pack_start (GTK_BOX (box), hbox, FALSE, FALSE, 0);

      label = gtk_label_new ("Time Real:");
      gtk_widget_show (label);
      gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
      gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);

      label = gtk_label_new ("");
      gtk_widget_show (label);
      gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
      proc->w_time_real = label;
      gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);

      item_list = proc->data;
      while (item_list)
	{

	  item_data = item_list->data;

	  hbox = gtk_hbox_new (FALSE, 0);
	  gtk_widget_show (hbox);
	  gtk_box_pack_start (GTK_BOX (box), hbox, FALSE, FALSE, 0);

	  label = gtk_label_new (item_data->text);
	  gtk_widget_show (label);
	  gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
	  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);

	  label = gtk_label_new (item_data->value);
	  gtk_widget_show (label);
	  gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
	  gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);

	  item_list = item_list->next;
	}

      hbox = gtk_hbox_new (FALSE, 0);
      gtk_widget_show (hbox);
      gtk_box_pack_start (GTK_BOX (box), hbox, FALSE, FALSE, 0);

      label = gtk_label_new ("Pid:");
      gtk_widget_show (label);
      gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
      gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);

      snprintf (s, sizeof (s), "%d", proc->pid);
      label = gtk_label_new (s);
      gtk_widget_show (label);
      gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
      gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
      proc->w_pid = label;

      hbox = gtk_hbox_new (FALSE, 0);
      gtk_widget_show (hbox);
      gtk_box_pack_start (GTK_BOX (box), hbox, FALSE, FALSE, 0);

      label = gtk_label_new ("Log:");
      gtk_widget_show (label);
      gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
      gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);

      label = gtk_entry_new ();
      gtk_widget_show (label);
      gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
      proc->w_log = label;

      hbox = gtk_hbox_new (TRUE, 0);
      gtk_widget_show (hbox);
      gtk_box_pack_start (GTK_BOX (box), hbox, FALSE, FALSE, 0);

      label = gtk_button_new_with_mnemonic ("Pause Job");
      gtk_widget_show (label);
      gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);

      g_signal_connect ((gpointer) label, "clicked",
			G_CALLBACK (proc_startstop), proc);

      label = gtk_button_new_with_mnemonic ("Kill Job");
      gtk_widget_show (label);
      gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);

      g_signal_connect ((gpointer) label, "clicked",
			G_CALLBACK (proc_kill), proc);

      n++;

      item = item->next;
    }

  gtk_widget_show_all (notebook);
}

static void
proc_free (struct renderix_data *data)
{
  GList *item;
  void *tmp;
  struct renderix_proc *proc;
  GList *item_list;
  struct renderix_item_data *item_data;

  DEBUG ("proc free");

  item = data->proc;
  while (item)
    {

      proc = item->data;
      timer_close (proc->time);

      tmp = proc->arg;

      while (*(proc->arg))
	{
	  free (*(proc->arg));
	  proc->arg++;
	}

      free (tmp);

      item_list = proc->data;
      while (item_list)
	{
	  item_data = item_list->data;
	  g_free (item_data->text);
	  g_free (item_data->value);
	  g_free (item_data);
	  item_list = item_list->next;
	}

      g_list_free (proc->data);

      item = item->next;
    }

  g_list_free (data->proc);
  data->proc = NULL;
}

gboolean
proc_timeout (struct renderix_data *data)
{
  GList *item;
  struct renderix_proc *proc;
  int ok = 1;
  char s[1024];

  DEBUG ("proc timeout");

  item = data->proc;
  while (item)
    {
      DEBUG ("proc timeout - update data");

      proc = item->data;

      gtk_label_set_text (GTK_LABEL (proc->w_time),
			  timer_get_time (proc->time));
      gtk_label_set_text (GTK_LABEL (proc->w_time_real),
			  timer_get_real (proc->time));

      switch (proc->status)
	{
	case 0:
	  timer_set_block (proc->time, FALSE);
	  timer_set_pause (proc->time, FALSE);

	  gtk_label_set_text (GTK_LABEL (proc->w_status), "RUNNING");

	  snprintf (s, sizeof (s), "%d", proc->pid);
	  gtk_label_set_text (GTK_LABEL (proc->w_pid), s);

	  if (!pthread_mutex_trylock (&proc->mutex))
	    {

	      if (*proc->buffer)
		{
		  if (!strncmp (proc->buffer, "Saved:", 6))
		    {
		      char *time;

		      time = g_strdup (timer_get_time (proc->time_image));
		      image_new (proc->buffer + 6, time, data);
		      g_free (time);

		      timer_close (proc->time_image);
		      proc->time_image = timer_new ();
		    }

		  gtk_entry_set_text (GTK_ENTRY (proc->w_log), proc->buffer);
		  proc->buffer[0] = 0;
		}

	      pthread_mutex_unlock (&proc->mutex);
	      pthread_cond_signal (&proc->cond);
	    }


	  ok = 0;

	  break;

	case 1:
	  if (proc->status == 1)
	    {
	      proc->status = 2;
	      pthread_join (proc->th, NULL);
	    }

	  timer_set_block (proc->time, TRUE);
	  timer_set_pause (proc->time, FALSE);

	  gtk_label_set_text (GTK_LABEL (proc->w_status), "QUITTING");
	  gtk_label_set_text (GTK_LABEL (proc->w_pid), "");

	  break;

	case 2:

	  timer_set_block (proc->time, TRUE);
	  timer_set_pause (proc->time, FALSE);

	  gtk_label_set_text (GTK_LABEL (proc->w_status), "QUIT");
	  gtk_label_set_text (GTK_LABEL (proc->w_pid), "");

	  break;

	case 3:

	  ok = 0;

	  timer_set_block (proc->time, FALSE);
	  timer_set_pause (proc->time, TRUE);

	  gtk_label_set_text (GTK_LABEL (proc->w_status), "PAUSED");
	  snprintf (s, sizeof (s), "%d", proc->pid);
	  gtk_label_set_text (GTK_LABEL (proc->w_pid), s);

	  break;
	}

      item = item->next;
    }

  if (ok)
    {
      DEBUG ("proc timeout - close data");

      gtk_widget_destroy (data->jobs);
      proc_free (data);
      gtk_widget_set_sensitive (data->frame, TRUE);
      return FALSE;
    }

  DEBUG ("proc timeout - close");
  return TRUE;
}

static void
proc_startstop (GtkWidget * w, struct renderix_proc *proc)
{
  DEBUG ("proc startstop");

  if (proc->status == 0)
    {
      proc->status = 3;
      gtk_button_set_label (GTK_BUTTON (w), "Start Job");
      kill (proc->pid, SIGSTOP);
    }
  else
    {
      proc->status = 0;
      gtk_button_set_label (GTK_BUTTON (w), "Pause Job");
      kill (proc->pid, SIGCONT);
    }
}

static void
proc_kill (GtkWidget * w, struct renderix_proc *proc)
{
  DEBUG ("proc  kill");

  kill (proc->pid, SIGQUIT);
}

static void
proc_killall (GtkWidget * w, struct renderix_data *data)
{
  struct renderix_proc *proc;
  GList *list;

  DEBUG ("proc  killall");

  list = data->proc;
  while (list)
    {
      proc = list->data;

      kill (proc->pid, SIGQUIT);

      list = list->next;
    }
}

static void
proc_pauseall (GtkWidget * w, struct renderix_data *data)
{
  struct renderix_proc *proc;
  static int i = 0;
  GList *list;

  DEBUG ("proc pauseall");

  if (!i)
    gtk_button_set_label (GTK_BUTTON (w), "Start all Job");
  else
    gtk_button_set_label (GTK_BUTTON (w), "Pause all Job");

  list = data->proc;
  while (list)
    {
      proc = list->data;

      if (!i)
	{
	  proc->status = 3;
	  kill (proc->pid, SIGSTOP);
	}
      else
	{
	  proc->status = 0;
	  kill (proc->pid, SIGCONT);
	}

      list = list->next;
    }

  if (!i)
    i = 1;
  else
    i = 0;
}

/* EOF */
