/* SomaX - Copyright (C) 2005 bakunin - Andrea Marchesini 
 *                                <bakunin@autistici.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __RENDERIX_H__
#define __RENDERIX_H__

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/wait.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <time.h>
#include <pthread.h>
#include <gtk/gtk.h>
#include <dirent.h>
#include <dlfcn.h>
#include "../mosix/libmosix.h"

#ifdef ENABLE_DEBUG
#  define DEBUG( x ) puts(x);
#else
#  define DEBUG( x ) 
#endif

struct renderix_timer_t {
  gint sec;
  gint real;

  gboolean paused;
  gboolean blocked;

} renderix_timer_t;

struct plugin_t
{
  gchar *name;
  gchar *author;
  gchar *description;
  gchar *version;
  gchar *licence;

  void *library;

  GtkWidget *(*widget)(void);
  GList *(*data)(void);

  gchar *file;
};

struct renderix_item_data {
  char *text;
  char *value;
} renderix_item_data;

struct renderix_item {
  char **arg;
  GList *data;
} renderix_item;

struct renderix_proc
{
  GList *data;
  char **arg;
  int pid;

  char buffer[1024];

  int status;
  int quit;

  struct renderix_timer_t *time;
  struct renderix_timer_t *time_image;

  pthread_t th;

  GtkWidget *w_status;
  GtkWidget *w_pid;
  GtkWidget *w_log;
  GtkWidget *w_time;
  GtkWidget *w_time_real;

  pthread_mutex_t mutex;
  pthread_cond_t cond;
};

struct renderix_data
{
  GtkWidget *jobs;
  GtkWidget *frame;
  GtkWidget *body;

  GtkWidget *images;

  struct plugin_t *plugin;

  GList *proc;
};

struct renderix_dialog_data
{
  GtkWidget *dialog;
  gint size;
};

extern char **env;
extern GList *session;
extern GtkWidget *notebook;

void dialog (char *);
int dialog_ask (char *text);

gboolean quit (GtkWidget *, void *, gpointer);

void *file_chooser (char *, int, int);

void proc_new (struct renderix_proc *);
void proc_window (struct renderix_data *);
gboolean proc_timeout (struct renderix_data *);

gboolean image_select (GtkWidget *, GdkEventButton *, gpointer);
void image_new (char *, char *, struct renderix_data *);

struct renderix_data *session_new(GtkWidget *, struct plugin_t *);
void session_close (GtkWidget *, struct renderix_data *);
void session_stop (GtkWidget *, struct renderix_data *);

void about_init(void);
void about_show(void);

gboolean openmosix_is_up(void);
gint openmosix_n_nodes(void);
void openmosix_stats_show(void);
void openmosix_stats(void);
void openmosix_graph_show(void);
void openmosix_graph(void);
void openmosix_init(void);
void *openmosix_thread(void *);
void openmosix_quit(void);

GtkWidget *graphic_new (void);
void graphic_set_range (GtkWidget *, gint);
void graphic_set_data (GtkWidget *, gint);
void graphic_set_color (GtkWidget *, gchar *);

void timer_init(void);
char *timer_get_real(struct renderix_timer_t *);
char *timer_get_time(struct renderix_timer_t *);
void timer_close(struct renderix_timer_t *);
void timer_destroy(void);
struct renderix_timer_t *timer_new(void);
void timer_set_block(struct renderix_timer_t *, gboolean);
void timer_set_pause(struct renderix_timer_t *, gboolean);

void signal_handler(int);
void signal_pipe(int);

#endif

/* EOF */
