/* SomaX - Copyright (C) 2005 bakunin - Andrea Marchesini 
 *                                <bakunin@autistici.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#else
# error Use configure; make; make install
#endif

#include "renderix.h"
#include "plugins/plugins.h"

static void session_start (GtkWidget *, struct renderix_data *);

struct renderix_data *
session_new (GtkWidget * w, struct plugin_t *plugin)
{
  GtkWidget *frame;
  GtkWidget *label;
  GtkWidget *tv;
  GtkWidget *sw;
  GtkListStore *model;
  GtkTreeSelection *selection;
  GtkCellRenderer *renderer;
  GtkWidget *button;
  GtkWidget *vbox;
  GtkWidget *box;
  GtkWidget *gbox;
  GtkWidget *image;
  gint i = 0;

  struct renderix_data *data;
  char s[1024];

  DEBUG ("proc session new");

  if (!plugin)
    {
      GList *list;

      list = plugin_get_list ();
      plugin = list->data;
    }

  data = (struct renderix_data *) g_malloc (sizeof (struct renderix_data));
  memset (data, 0, sizeof (struct renderix_data));

  data->plugin = plugin;

  session = g_list_append (session, data);

  gbox = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (gbox);

  snprintf (s, sizeof (s), "Session %s", plugin->name);
  label = gtk_label_new (s);
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (gbox), label, TRUE, TRUE, 0);

  label = gtk_button_new ();
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (gbox), label, TRUE, TRUE, 0);
  gtk_button_set_relief (GTK_BUTTON (label), GTK_RELIEF_NONE);

  image = gtk_image_new_from_stock ("gtk-close", GTK_ICON_SIZE_MENU);
  gtk_widget_show (image);
  gtk_container_add (GTK_CONTAINER (label), image);

  g_signal_connect ((gpointer) label, "clicked", G_CALLBACK (session_close),
		    data);

  box = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (box);

  i = gtk_notebook_append_page (GTK_NOTEBOOK (notebook), box, gbox);
  gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook), i);

  gbox = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (gbox);
  gtk_box_pack_start (GTK_BOX (box), gbox, TRUE, TRUE, 0);
  data->body = gbox;

  sw = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_set_size_request (sw, 200, -1);
  gtk_widget_show (sw);
  gtk_box_pack_start (GTK_BOX (box), sw, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
				  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  model = gtk_list_store_new (3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
  tv = gtk_tree_view_new_with_model (GTK_TREE_MODEL (model));
  gtk_tree_view_set_enable_search (GTK_TREE_VIEW (tv), FALSE);
  gtk_container_add (GTK_CONTAINER (sw), tv);
  gtk_widget_show (tv);
  g_signal_connect (GTK_OBJECT (tv), "button-press-event",
		    GTK_SIGNAL_FUNC (image_select), NULL);

  data->images = tv;

  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tv));
  gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
  gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (tv), TRUE);

  g_object_unref (model);

  renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (tv),
					       -1, "Images List", renderer,
					       "text", 1, NULL);

  renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (tv),
					       -1, "Times", renderer,
					       "text", 2, NULL);

  vbox = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox);
  gtk_box_pack_start (GTK_BOX (gbox), vbox, TRUE, TRUE, 0);
  data->frame = vbox;

  frame = plugin->widget ();
  gtk_widget_show (frame);
  gtk_box_pack_start (GTK_BOX (vbox), frame, TRUE, TRUE, 0);

  box = gtk_hbox_new (TRUE, 0);
  gtk_widget_show (box);
  gtk_box_pack_start (GTK_BOX (vbox), box, FALSE, FALSE, 0);

  button = gtk_button_new_from_stock ("gtk-close");
  gtk_widget_show (button);
  gtk_box_pack_start (GTK_BOX (box), button, TRUE, TRUE, 0);

  g_signal_connect ((gpointer) button, "clicked", G_CALLBACK (session_close),
		    data);

  button = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (button);
  gtk_box_pack_start (GTK_BOX (box), button, TRUE, TRUE, 0);

  g_signal_connect ((gpointer) button, "clicked", G_CALLBACK (session_start),
		    data);

  return data;
}

static void
session_start (GtkWidget * widget, struct renderix_data *data)
{
  GtkListStore *store;
  GtkTreeIter iter;
  struct renderix_proc *proc;
  struct renderix_item *item;
  GList *cmd, *old;

  DEBUG ("session start");

  store =
    (GtkListStore *) gtk_tree_view_get_model (GTK_TREE_VIEW (data->images));

  while (gtk_tree_model_iter_nth_child
	 (GTK_TREE_MODEL (store), &iter, NULL, 0))
    gtk_list_store_remove (store, &iter);

  if ((cmd = data->plugin->data ()))
    {

      old = cmd;

      while (cmd)
	{
	  proc =
	    (struct renderix_proc *) g_malloc (sizeof (struct renderix_proc));
	  memset (proc, 0, sizeof (struct renderix_proc));

	  item = cmd->data;
	  proc->arg = item->arg;
	  proc->data = item->data;
	  proc_new (proc);

	  data->proc = g_list_append (data->proc, proc);

	  free (item);

	  cmd = cmd->next;
	}

      g_list_free (old);

      proc_window (data);

      g_timeout_add (200, (GSourceFunc) proc_timeout, data);
      gtk_widget_set_sensitive (data->frame, FALSE);
    }
}

void
session_stop (GtkWidget * w, struct renderix_data *data)
{
  struct renderix_proc *proc;
  GList *list;

  DEBUG ("session stop");

  list = data->proc;
  while (list)
    {
      proc = list->data;

      kill (proc->pid, SIGQUIT);

      list = list->next;
    }
}

void
session_close (GtkWidget * w, struct renderix_data *data)
{
  GList *t;

  DEBUG ("session close");

  if (data->proc
      && dialog_ask ("Working progress... Sure to close this session?") ==
      GTK_RESPONSE_OK)
    session_stop (w, data);
  else if (data->proc)
    return;

  if ((t = g_list_find (session, data)))
    {

      gtk_notebook_remove_page (GTK_NOTEBOOK (notebook),
				g_list_position (session, t));
      session = g_list_remove (session, data);
    }

  if (!g_list_length (session))
    {
      if (GTK_IS_WIDGET (w))
	gtk_widget_hide (gtk_widget_get_toplevel (w));
      gtk_main_quit ();
    }
}

/* EOF */
