/* SomaX - Copyright (C) 2005 bakunin - Andrea Marchesini 
 *                                <bakunin@autistici.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#else
# error Use configure; make; make install
#endif

#include "renderix.h"

static GList *timer_list = NULL;

struct renderix_timer_t *
timer_new (void)
{
  struct renderix_timer_t *t;

  DEBUG ("timer_new");

  t = g_malloc (sizeof (struct renderix_timer_t));
  memset (t, 0, sizeof (struct renderix_timer_t));

  timer_list = g_list_append (timer_list, t);

  return t;
}

void
timer_close (struct renderix_timer_t *t)
{
  DEBUG ("timer_close");

  timer_list = g_list_remove (timer_list, t);

  free (t);
}

static gint
timer_timeout (gpointer dummy)
{
  GList *t;
  struct renderix_timer_t *tmp;

  DEBUG ("timer_timeout - start");

  t = timer_list;
  while (t)
    {
      tmp = t->data;

      if (tmp->blocked == FALSE)
	{
	  if (tmp->paused == FALSE)
	    tmp->sec++;
	  tmp->real++;
	}
      t = t->next;
    }

  DEBUG ("timer_timeout - end");

  return TRUE;
}

char *
timer_get_time (struct renderix_timer_t *t)
{
  static char s[1024];
  gint sec;
  gint min;
  gint hour;

  DEBUG ("timer_get_time");

  sec = t->sec % 60;
  min = t->sec / 60;
  hour = min / 60;
  min = min % 60;

  snprintf (s, sizeof (s), "%d:%.2d:%.2d", hour, min, sec);

  return s;
}

char *
timer_get_real (struct renderix_timer_t *t)
{
  static char s[1024];
  gint sec;
  gint min;
  gint hour;

  DEBUG ("timer_get_real");

  sec = t->real % 60;
  min = t->real / 60;
  hour = min / 60;
  min = min % 60;

  snprintf (s, sizeof (s), "%d:%.2d:%.2d", hour, min, sec);

  return s;
}

void
timer_init (void)
{
  DEBUG ("timer_init");

  g_timeout_add (1000, (GSourceFunc) timer_timeout, NULL);
}

void
timer_set_block (struct renderix_timer_t *t, gboolean v)
{
  DEBUG ("timer_set_block");

  t->blocked = v;
}

void
timer_set_pause (struct renderix_timer_t *t, gboolean v)
{
  DEBUG ("timer_set_pause");

  t->paused = v;
}

void
timer_destroy (void)
{
  GList *t = timer_list;

  DEBUG ("timer_destroy");

  while (t)
    {
      free (t->data);
      t = t->next;
    }

  g_list_free (timer_list);

  timer_list = NULL;
}

/* EOF */
